@@ -303,73 +303,82 @@ CODE_FRAGMENT
  * 	in styp_to_sec_flags().
  */
 static long
-DEFUN(sec_to_styp_flags, (sec_name, sec_flags),
-	CONST char *		sec_name	AND
-	flagword	sec_flags)
+sec_to_styp_flags (sec_name, sec_flags)
+     CONST char *sec_name;
+     flagword sec_flags;
 {
   long styp_flags = 0;
 
   if (!strcmp(sec_name, _TEXT)) 
-  {
-    styp_flags = STYP_TEXT;
-  } 
+    {
+      styp_flags = STYP_TEXT;
+    }
   else if (!strcmp(sec_name, _DATA)) 
-  {
-    styp_flags = STYP_DATA;
+    {
+      styp_flags = STYP_DATA;
 #ifdef TWO_DATA_SECS
-  }
+    }
   else if (!strcmp(sec_name, ".data2")) 
-  {
-    styp_flags = STYP_DATA;
+    {
+      styp_flags = STYP_DATA;
 #endif				/* TWO_DATA_SECS */
-  }
+    }
   else if (!strcmp(sec_name, _BSS)) 
-  {
-    styp_flags = STYP_BSS;
+    {
+      styp_flags = STYP_BSS;
 #ifdef _COMMENT
-  } 
+    } 
   else if (!strcmp(sec_name, _COMMENT)) 
-  {
-    styp_flags = STYP_INFO;
+    {
+      styp_flags = STYP_INFO;
 #endif				/* _COMMENT */
 #ifdef _LIB
-  }
+    }
   else if (!strcmp(sec_name, _LIB)) 
-  {
-    styp_flags = STYP_LIB;
+    {
+      styp_flags = STYP_LIB;
 #endif				/* _LIB */
 #ifdef _LIT
-  }
+    }
   else if (!strcmp (sec_name, _LIT))
-  {
-    styp_flags = STYP_LIT;
+    {
+      styp_flags = STYP_LIT;
 #endif /* _LIT */
-
-  }
+    }
+  else if (!strcmp(sec_name, ".debug"))
+    {
+      styp_flags = STYP_INFO;
+    }
+  else if (!strcmp(sec_name, ".stab")
+ 	   || !strncmp(sec_name, ".stabstr", 8))
+    {
+      styp_flags = STYP_INFO;
+    }
   /* Try and figure out what it should be */
   else if (sec_flags & SEC_CODE) 
-  {
-    styp_flags = STYP_TEXT;
-  }
+    {
+      styp_flags = STYP_TEXT;
+    }
   else  if (sec_flags & SEC_DATA) 
-  {
-    styp_flags = STYP_DATA;
-  }
+    {
+      styp_flags = STYP_DATA;
+    }
   else if (sec_flags & SEC_READONLY)
-  {
+    {
 #ifdef STYP_LIT			/* 29k readonly text/data section */
-    styp_flags = STYP_LIT;
+      styp_flags = STYP_LIT;
 #else
-    styp_flags = STYP_TEXT;
+      styp_flags = STYP_TEXT;
 #endif				/* STYP_LIT */
-  }
+    }
   else if (sec_flags & SEC_LOAD)
-  {
-    styp_flags = STYP_TEXT;
-  }
-  else {
-     styp_flags = STYP_BSS;
-   }
+    {
+      styp_flags = STYP_TEXT;
+    }
+  else if (sec_flags & SEC_ALLOC)
+    {
+      styp_flags = STYP_BSS;
+    }
 
 #ifdef STYP_NOLOAD
   if (sec_flags & SEC_NEVER_LOAD)
@@ -427,9 +436,13 @@ DEFUN(styp_to_sec_flags, (abfd, hdr),
       sec_flags |= SEC_ALLOC;
   }
   else if (styp_flags & STYP_INFO) 
-  {
-    /* Nothing to do.  */
-  }
+    {
+      /* This should be marked as SEC_DEBUGGING, but that can't be
+	 done until we make sure that strip can still work.  strip
+	 will probably have to preserve the same number of sections to
+	 ensure that the section vma matches the section file
+	 position.  */
+    }
   else
   {
     sec_flags |= SEC_ALLOC | SEC_LOAD;
@@ -451,7 +464,7 @@ DEFUN(styp_to_sec_flags, (abfd, hdr),
   return(sec_flags);
 }
 
-#define	get_index(symbol)	((int) (symbol)->udata)
+#define	get_index(symbol)	((long) (symbol)->udata)
 
 /*
 INTERNAL_DEFINITION
@@ -574,6 +587,12 @@ dependent COFF routines
 .       bfd_byte *data,
 .       unsigned int *src_ptr,
 .       unsigned int *dst_ptr));
+. int (*_bfd_coff_reloc16_estimate) PARAMS ((
+.       asection *input_section,
+.       asymbol **symbols,
+.       arelent *r,
+.       unsigned int shrink));	
+.
 .} bfd_coff_backend_data;
 .
 .#define coff_backend_info(abfd) ((bfd_coff_backend_data *) (abfd)->xvec->backend_data)
@@ -650,6 +669,10 @@ dependent COFF routines
 .#define bfd_coff_reloc16_extra_cases(abfd, seclet, reloc, data, src_ptr, dst_ptr)\
 .        ((coff_backend_info (abfd)->_bfd_coff_reloc16_extra_cases)\
 .         (abfd, seclet, reloc, data, src_ptr, dst_ptr))
+.
+.#define bfd_coff_reloc16_estimate(abfd, section, symbols, reloc, shrink)\
+.        ((coff_backend_info (abfd)->_bfd_coff_reloc16_estimate)\
+.         (section, symbols, reloc, shrink))
 . 
 */
 
@@ -688,6 +711,8 @@ DEFUN (coff_make_section_hook, (abfd, name),
        char *name)
 {
 #ifdef TWO_DATA_SECS
+  /* FIXME: This predates the call to bfd_make_section_anyway
+     in make_a_section_from_file, and can probably go away.  */
   /* On SCO a file created by the Microsoft assembler can have two
      .data sections.  We use .data2 for the second one.  */
   if (strcmp (name, _DATA) == 0)
@@ -806,9 +831,9 @@ DEFUN(coff_mkobject_hook,(abfd, filehdr, aouthdr),
    at linking together COFF files for different architectures.  */
 
 static boolean
-DEFUN (coff_set_arch_mach_hook, (abfd, filehdr),
-       bfd *abfd AND
-       PTR filehdr)
+coff_set_arch_mach_hook(abfd, filehdr)
+     bfd *abfd;
+     PTR filehdr;
 {
   long machine;
   enum bfd_architecture arch;
@@ -818,9 +843,9 @@ DEFUN (coff_set_arch_mach_hook, (abfd, filehdr),
   switch (internal_f->f_magic) {
 #ifdef I386MAGIC
   case I386MAGIC:
-#ifdef I386AIXMAGIC
+  case I386PTXMAGIC:
   case I386AIXMAGIC:		/* Danbury PS/2 AIX C Compiler */
-#endif
+  case LYNXCOFFMAGIC:		/* shadows the m68k Lynx number below, sigh */
     arch = bfd_arch_i386;
     machine = 0;
     break;
@@ -837,6 +862,15 @@ DEFUN (coff_set_arch_mach_hook, (abfd, filehdr),
 #ifdef MC68MAGIC
   case MC68MAGIC:
   case M68MAGIC:
+#ifdef MC68KBCSMAGIC
+  case MC68KBCSMAGIC:
+#endif
+#ifdef APOLLOM68KMAGIC
+  case APOLLOM68KMAGIC:
+#endif
+#ifdef LYNXCOFFMAGIC
+  case LYNXCOFFMAGIC:
+#endif
     arch = bfd_arch_m68k;
     machine = 68020;
     break;
@@ -915,7 +949,16 @@ DEFUN (coff_set_arch_mach_hook, (abfd, filehdr),
 #ifdef H8300MAGIC
   case H8300MAGIC:
     arch = bfd_arch_h8300;
-    machine = 0;
+    machine = bfd_mach_h8300;
+    /* !! FIXME this probably isn't the right place for this */
+    abfd->flags |= BFD_IS_RELAXABLE;
+    break;
+#endif
+
+#ifdef H8300HMAGIC
+  case H8300HMAGIC:
+    arch = bfd_arch_h8300;
+    machine = bfd_mach_h8300h;
     /* !! FIXME this probably isn't the right place for this */
     abfd->flags |= BFD_IS_RELAXABLE;
     break;
@@ -935,6 +978,13 @@ DEFUN (coff_set_arch_mach_hook, (abfd, filehdr),
     break;
 #endif
 
+#ifdef SPARCMAGIC
+  case SPARCMAGIC:
+    arch = bfd_arch_sparc;
+    machine = 0;
+    break;
+#endif
+
   default:			/* Unreadable input file type */
     arch = bfd_arch_obscure;
 break;
@@ -1102,12 +1152,24 @@ DEFUN(coff_set_flags,(abfd, magicp, flagsp),
 #ifdef I386MAGIC
   case bfd_arch_i386:
     *magicp = I386MAGIC;
+#ifdef LYNXOS
+    /* Just overwrite the usual value if we're doing Lynx. */
+    *magicp = LYNXCOFFMAGIC;
+#endif
     return true;
     break;
 #endif
 #ifdef MC68MAGIC
   case bfd_arch_m68k:
+#ifdef APOLLOM68KMAGIC
+    *magicp = APOLLO_COFF_VERSION_NUMBER;
+#else
     *magicp = MC68MAGIC;
+#endif
+#ifdef LYNXOS
+    /* Just overwrite the usual value if we're doing Lynx. */
+    *magicp = LYNXCOFFMAGIC;
+#endif
     return true;
     break;
 #endif
@@ -1120,8 +1182,15 @@ DEFUN(coff_set_flags,(abfd, magicp, flagsp),
 #endif
 #ifdef H8300MAGIC
     case bfd_arch_h8300:
-      *magicp = H8300MAGIC;
-      return true;
+    switch (bfd_get_mach (abfd)) 
+      {
+      case bfd_mach_h8300:
+	*magicp = H8300MAGIC;
+	return true;
+      case bfd_mach_h8300h:
+	*magicp = H8300HMAGIC;
+	return true;
+      }
       break;
 #endif
 
@@ -1132,6 +1201,17 @@ DEFUN(coff_set_flags,(abfd, magicp, flagsp),
       break;
 #endif
 
+#ifdef SPARCMAGIC
+  case bfd_arch_sparc:
+    *magicp = SPARCMAGIC;
+#ifdef LYNXOS
+    /* Just overwrite the usual value if we're doing Lynx. */
+    *magicp = LYNXCOFFMAGIC;
+#endif
+    return true;
+    break;
+#endif
+
 #ifdef H8500MAGIC
     case bfd_arch_h8500:
       *magicp = H8500MAGIC;
@@ -1296,6 +1376,10 @@ coff_add_missing_symbols (abfd)
 	 continue;
 	if (!strcmp (name, _TEXT))
 	 need_text = 0;
+#ifdef APOLLO_M68
+	else if (!strcmp(name, ".wtext"))
+	  need_text = 0;
+#endif
 	else if (!strcmp (name, _DATA))
 	 need_data = 0;
 	else if (!strcmp (name, _BSS))
@@ -1589,6 +1673,11 @@ DEFUN(coff_write_object_contents,(abfd),
   internal_a.magic = PAGEMAGICBCS;
 #endif				/* M88 */
 
+#if APOLLO_M68
+#define __A_MAGIC_SET__
+  internal_a.magic = APOLLO_COFF_VERSION_NUMBER;
+#endif
+
 #if M68 || WE32K
 #define __A_MAGIC_SET__
   /* Never was anything here for the 68k */
@@ -1857,7 +1946,7 @@ DEFUN(coff_slurp_symbol_table,(abfd),
       /*
 	We use the native name field to point to the cached field
 	*/
-      src->u.syment._n._n_n._n_zeroes = (int) dst;
+      src->u.syment._n._n_n._n_zeroes = (long) dst;
       dst->symbol.section = coff_section_from_bfd_index(abfd,
 							src->u.syment.n_scnum);
       dst->symbol.flags = 0;
@@ -2062,8 +2151,9 @@ SUBSUBSECTION
 
 #ifndef CALC_ADDEND
 #define CALC_ADDEND(abfd, ptr, reloc, cache_ptr) 	\
-	    if (ptr && bfd_asymbol_bfd(ptr) == abfd		\
-		&& ((ptr->flags & BSF_OLD_COMMON)== 0))	\
+	    if (ptr && bfd_asymbol_bfd(ptr) == abfd	\
+		&& !bfd_is_com_section(ptr->section)	\
+		&& !(ptr->flags & BSF_OLD_COMMON))	\
 	    {						\
 		cache_ptr->addend = -(ptr->section->vma + ptr->value);	\
 	    }						\
@@ -2222,6 +2312,21 @@ bfd *abfd;
   }
 #endif
 
+#ifndef coff_reloc16_estimate
+#define coff_reloc16_estimate dummy_reloc16_estimate
+
+static int
+dummy_reloc16_estimate(input_section, symbols, reloc, shrink)
+     asection *input_section;
+     asymbol **symbols;
+     arelent *reloc;
+     unsigned int shrink;
+{
+  abort ();
+}
+
+#endif
+
 #ifndef coff_reloc16_extra_cases
 #define coff_reloc16_extra_cases dummy_reloc16_extra_cases
 /* This works even if abort is not declared in any header file.  */
@@ -2234,6 +2339,7 @@ dummy_reloc16_extra_cases (abfd, seclet, reloc, data, src_ptr, dst_ptr)
      unsigned int *src_ptr;
      unsigned int *dst_ptr;
 {
+  fprintf(stderr, "%s\n", reloc->howto->name);
   abort ();
 }
 #endif
@@ -2253,7 +2359,8 @@ static CONST bfd_coff_backend_data bfd_coff_std_swap_table = {
  coff_swap_filehdr_in, coff_swap_aouthdr_in, coff_swap_scnhdr_in,
  coff_bad_format_hook, coff_set_arch_mach_hook, coff_mkobject_hook,
  styp_to_sec_flags, coff_make_section_hook, coff_set_alignment_hook,
- coff_slurp_symbol_table, symname_in_debug_hook, coff_reloc16_extra_cases
+ coff_slurp_symbol_table, symname_in_debug_hook, 
+ coff_reloc16_extra_cases, coff_reloc16_estimate
 };
 
 #define coff_core_file_failing_command	_bfd_dummy_core_file_failing_command
