@@ -26,7 +26,7 @@ SECTION
 	symbols; they are left alone until required, then read in
 	en-mass and traslated into an internal form. There is a common
 	routine <<bfd_perform_relocation>> which acts upon the
-	canonical form to to the actual fixup.
+	canonical form to do the actual fixup.
 
 	Note that relocations are maintained on a per section basis,
 	whilst symbols are maintained on a per BFD basis.
@@ -65,7 +65,7 @@ CODE_FRAGMENT
 .       {* The relocation was performed, but there was an overflow. *}
 .  bfd_reloc_overflow,
 .
-.       {* The address to relocate was not within the section supplied*}
+.       {* The address to relocate was not within the section supplied. *}
 .  bfd_reloc_outofrange,
 .
 .       {* Used by special functions *}
@@ -74,10 +74,10 @@ CODE_FRAGMENT
 .       {* Unused *}
 .  bfd_reloc_notsupported,
 .
-.       {* Unsupported relocation size requested.  *}
+.       {* Unsupported relocation size requested. *}
 .  bfd_reloc_other,
 .
-.       {* The symbol to relocate against was undefined.*}
+.       {* The symbol to relocate against was undefined. *}
 .  bfd_reloc_undefined,
 .
 .       {* The relocation was performed, but may not be ok - presently
@@ -94,7 +94,7 @@ CODE_FRAGMENT
 .  struct symbol_cache_entry **sym_ptr_ptr;
 .
 .       {* offset in section *}
-.  rawdata_offset address;
+.  bfd_size_type address;
 .
 .       {* addend for relocation value *}
 .  bfd_vma addend;    
@@ -242,6 +242,34 @@ DESCRIPTION
         
 */
 
+/*
+SUBSUBSECTION
+	<<enum complain_overflow>>
+
+	Indicates what sort of overflow checking should be done when
+	performing a relocation.
+
+CODE_FRAGMENT
+.
+.enum complain_overflow
+.{
+.	{* Do not complain on overflow. *}
+.  complain_overflow_dont,
+.
+.	{* Complain if the bitfield overflows, whether it is considered
+.	   as signed or unsigned. *}
+.  complain_overflow_bitfield,
+.
+.	{* Complain if the value overflows when considered as signed
+.	   number. *}
+.  complain_overflow_signed,
+.
+.	{* Complain if the value overflows when considered as an
+.	   unsigned number. *}
+.  complain_overflow_unsigned
+.};
+
+*/
 
 /*
 SUBSUBSECTION 
@@ -267,12 +295,21 @@ CODE_FRAGMENT
 .           unwanted data from the relocation.  *}
 .  unsigned int rightshift;
 .
-.       {*  The size of the item to be relocated - 0, is one byte, 1 is 2
-.           bytes, 2 is four bytes.  A negative value indicates that the
-.	    result is to be subtracted from the data.  *}
+.	{*  The size of the item to be relocated.  This is *not* a
+.	    power-of-two measure.
+.		 0 : one byte
+.		 1 : two bytes
+.		 2 : four bytes
+.		 3 : nothing done (unless special_function is nonzero)
+.		 4 : eight bytes
+.		-2 : two bytes, result should be subtracted from the
+.		     data instead of added
+.	    There is currently no trivial way to extract a "number of
+.	    bytes" from a howto pointer.  *}
 .  int size;
 .
-.       {*  Now obsolete?  But m68k-coff still uses it... *}
+.       {*  The number of bits in the item to be relocated.  This is used
+.	    when doing overflow checking.  *}
 .  unsigned int bitsize;
 .
 .       {*  Notes that the relocation is relative to the location in the
@@ -281,14 +318,13 @@ CODE_FRAGMENT
 .           being relocated. *}
 .  boolean pc_relative;
 .
+.	{*  The bit position of the reloc value in the destination.
+.	    The relocated value is left shifted by this amount. *}
 .  unsigned int bitpos;
 .
-.       {*  Now obsolete *}
-.  boolean absolute;
-.
-.       {* Causes the relocation routine to return an error if overflow
-.          is detected when relocating. *}
-.  boolean complain_on_overflow;
+.	{* What type of overflow error should be checked for when
+.	   relocating. *}
+.  enum complain_overflow complain_on_overflow;
 .
 .       {* If this field is non null, then the supplied function is
 .          called rather than the normal function. This allows really
@@ -316,13 +352,13 @@ CODE_FRAGMENT
 .          sun4 extended relocs, the value in the offset part of a
 .          relocating field is garbage so we never use it. In this case
 .          the mask would be 0x00000000. *}
-.  bfd_word src_mask;
+.  bfd_vma src_mask;
 .
 .       {* The dst_mask is what parts of the instruction are replaced
 .          into the instruction. In most cases src_mask == dst_mask,
 .          except in the above special case, where dst_mask would be
 .          0x000000ff, and src_mask would be 0x00000000.   *}
-.  bfd_word dst_mask;           
+.  bfd_vma dst_mask;           
 .
 .       {* When some formats create PC relative instructions, they leave
 .          the value of the pc of the place being relocated in the offset
@@ -344,15 +380,15 @@ DESCRIPTION
 	The HOWTO define is horrible and will go away.
 
 
-.#define HOWTO(C, R,S,B, P, BI, ABS, O, SF, NAME, INPLACE, MASKSRC, MASKDST, PC) \
-.  {(unsigned)C,R,S,B, P, BI, ABS,O,SF,NAME,INPLACE,MASKSRC,MASKDST,PC}
+.#define HOWTO(C, R,S,B, P, BI, O, SF, NAME, INPLACE, MASKSRC, MASKDST, PC) \
+.  {(unsigned)C,R,S,B, P, BI, O,SF,NAME,INPLACE,MASKSRC,MASKDST,PC}
 
 DESCRIPTION
 	And will be replaced with the totally magic way. But for the
 	moment, we are compatible, so do it this way..
 
 
-.#define NEWHOWTO( FUNCTION, NAME,SIZE,REL,IN) HOWTO(0,0,SIZE,0,REL,0,false,false,FUNCTION, NAME,false,0,0,IN)
+.#define NEWHOWTO( FUNCTION, NAME,SIZE,REL,IN) HOWTO(0,0,SIZE,0,REL,0,complain_overflow_dont,FUNCTION, NAME,false,0,0,IN)
 .
 DESCRIPTION
 	Helper routine to turn a symbol into a relocation value.
@@ -434,7 +470,7 @@ DEFUN(bfd_perform_relocation,(abfd,
 {
   bfd_vma relocation;
   bfd_reloc_status_type flag = bfd_reloc_ok;
-  bfd_vma addr = reloc_entry->address ;
+  bfd_size_type addr = reloc_entry->address ;
   bfd_vma output_base = 0;
   reloc_howto_type *howto = reloc_entry->howto;
   asection *reloc_target_output_section ;
@@ -444,136 +480,322 @@ DEFUN(bfd_perform_relocation,(abfd,
   symbol = *( reloc_entry->sym_ptr_ptr);
   if ((symbol->section == &bfd_abs_section) 
       && output_bfd != (bfd *)NULL) 
-  {
-    reloc_entry->address += input_section->output_offset;
-       
-    return bfd_reloc_ok;
-       
-  }
+    {
+      reloc_entry->address += input_section->output_offset;
+      return bfd_reloc_ok;
+    }
 
-  if ((symbol->section == &bfd_und_section) && output_bfd == (bfd *)NULL) {
+  /* If we are not producing relocateable output, return an error if
+     the symbol is not defined.  An undefined weak symbol is
+     considered to have a value of zero (SVR4 ABI, p. 4-27).  */
+  if (symbol->section == &bfd_und_section
+      && (symbol->flags & BSF_WEAK) == 0
+      && output_bfd == (bfd *) NULL)
     flag = bfd_reloc_undefined;
-  }
 
-  if (howto->special_function) {
-    bfd_reloc_status_type cont;
-    cont = howto->special_function(abfd,
-				   reloc_entry,
-				   symbol,
-				   data,
-				   input_section,
-				   output_bfd);
-    if (cont != bfd_reloc_continue) return cont;
-  }
+  /* If there is a function supplied to handle this relocation type,
+     call it.  It'll return `bfd_reloc_continue' if further processing
+     can be done.  */
+  if (howto->special_function)
+    {
+      bfd_reloc_status_type cont;
+      cont = howto->special_function (abfd, reloc_entry, symbol, data,
+				      input_section, output_bfd);
+      if (cont != bfd_reloc_continue)
+	return cont;
+    }
 
-  /* 
-    Work out which section the relocation is targetted at and the
-    initial relocation command value.
-    */
+  /* Is the address of the relocation really within the section?  */
+  if (reloc_entry->address > input_section->_cooked_size)
+    return bfd_reloc_outofrange;
 
+  /* Work out which section the relocation is targetted at and the
+     initial relocation command value.  */
 
-  if (bfd_is_com_section (symbol->section)) {
+  /* Get symbol value.  (Common symbols are special.)  */
+  if (bfd_is_com_section (symbol->section))
     relocation = 0;
-  }
-  else {
+  else
     relocation = symbol->value;
-  }
 
 
   reloc_target_output_section = symbol->section->output_section;
 
-  if (output_bfd && howto->partial_inplace==false) {
+  /* Convert input-section-relative symbol value to absolute.  */
+  if (output_bfd && howto->partial_inplace==false)
     output_base = 0;
-  }
-  else {
+  else
     output_base = reloc_target_output_section->vma;
 
-  }
-
   relocation += output_base + symbol->section->output_offset;
 
+  /* Add in supplied addend.  */
   relocation += reloc_entry->addend;
 
-  if(reloc_entry->address > input_section->_cooked_size)
-  {
-    return bfd_reloc_outofrange;
-  }
-          
+  /* Here the variable relocation holds the final address of the
+     symbol we are relocating against, plus any addend.  */
 
   if (howto->pc_relative == true)
-  {
-    /*
-      Anything which started out as pc relative should end up that
-      way too. 
-      
-      There are two ways we can see a pcrel instruction. Sometimes
-      the pcrel displacement has been partially calculated, it
-      includes the distance from the start of the section to the
-      instruction in it (e.g., sun3), and sometimes the field is
-      totally blank - e.g., m88kbcs.
-      */
+    {
+      /* This is a PC relative relocation.  We want to set RELOCATION
+	 to the distance between the address of the symbol and the
+	 location.  RELOCATION is already the address of the symbol.
 
-        
-    relocation -= 
-     input_section->output_section->vma + input_section->output_offset;
+	 We start by subtracting the address of the section containing
+	 the location.
 
-    if (howto->pcrel_offset == true) {
-      relocation -= reloc_entry->address;
+	 If pcrel_offset is set, we must further subtract the position
+	 of the location within the section.  Some targets arrange for
+	 the addend to be the negative of the position of the location
+	 within the section; for example, i386-aout does this.  For
+	 i386-aout, pcrel_offset is false.  Some other targets do not
+	 include the position of the location; for example, m88kbcs,
+	 or ELF.  For those targets, pcrel_offset is true.
+
+	 If we are producing relocateable output, then we must ensure
+	 that this reloc will be correctly computed when the final
+	 relocation is done.  If pcrel_offset is false we want to wind
+	 up with the negative of the location within the section,
+	 which means we must adjust the existing addend by the change
+	 in the location within the section.  If pcrel_offset is true
+	 we do not want to adjust the existing addend at all.
+
+	 FIXME: This seems logical to me, but for the case of
+	 producing relocateable output it is not what the code
+	 actually does.  I don't want to change it, because it seems
+	 far too likely that something will break.  */
+
+      relocation -= 
+	input_section->output_section->vma + input_section->output_offset;
+
+      if (howto->pcrel_offset == true)
+	relocation -= reloc_entry->address;
     }
 
-  }
-
   if (output_bfd!= (bfd *)NULL) 
-  {
-    if ( howto->partial_inplace == false)  
     {
-      /*
-	This is a partial relocation, and we want to apply the relocation
-	to the reloc entry rather than the raw data. Modify the reloc
-	inplace to reflect what we now know.
-	*/
-      reloc_entry->addend = relocation  ;
-      reloc_entry->address +=  input_section->output_offset;
-      return flag;
-    }
-    else 
-    {
-      /* This is a partial relocation, but inplace, so modify the
-	 reloc record a bit. 
-	 
-	 If we've relocated with a symbol with a section, change
-	 into a ref to  the section belonging to the symbol
-	 */
-
-      reloc_entry->address += input_section->output_offset;
-
-      if (abfd->xvec->flavour == bfd_target_coff_flavour) 
-      {
-	relocation -= reloc_entry->addend;
-	reloc_entry->addend = 0;
-      }
+      if ( howto->partial_inplace == false)  
+	{
+	  /* This is a partial relocation, and we want to apply the relocation
+	     to the reloc entry rather than the raw data. Modify the reloc
+	     inplace to reflect what we now know.  */
+	  reloc_entry->addend = relocation;
+	  reloc_entry->address +=  input_section->output_offset;
+	  return flag;
+	}
       else
-      {
-	reloc_entry->addend = relocation  ;
-      }
-    }
-  }
-  else 
-  {
-    reloc_entry->addend = 0;
-  }
-  
+	{
+	  /* This is a partial relocation, but inplace, so modify the
+	     reloc record a bit. 
 
+	     If we've relocated with a symbol with a section, change
+	     into a ref to the section belonging to the symbol.  */
+
+	  reloc_entry->address += input_section->output_offset;
+
+	  /* WTF?? */
+	  if (abfd->xvec->flavour == bfd_target_coff_flavour) 
+	    {
+#if 1
+	      /* For m68k-coff, the addend was being subtracted twice during
+		 relocation with -r.  Removing the line below this comment
+		 fixes that problem; see PR 2953.
+
+However, Ian wrote the following, regarding removing the line below,
+which explains why it is still enabled:  --djm
+
+If you put a patch like that into BFD you need to check all the COFF
+linkers.  I am fairly certain that patch will break coff-i386 (e.g.,
+SCO); see coff_i386_reloc in coff-i386.c where I worked around the
+problem in a different way.  There may very well be a reason that the
+code works as it does.
+
+Hmmm.  The first obvious point is that bfd_perform_relocation should
+not have any tests that depend upon the flavour.  It's seem like
+entirely the wrong place for such a thing.  The second obvious point
+is that the current code ignores the reloc addend when producing
+relocateable output for COFF.  That's peculiar.  In fact, I really
+have no idea what the point of the line you want to remove is.
+
+A typical COFF reloc subtracts the old value of the symbol and adds in
+the new value to the location in the object file (if it's a pc
+relative reloc it adds the difference between the symbol value and the
+location).  When relocating we need to preserve that property.
+
+BFD handles this by setting the addend to the negative of the old
+value of the symbol.  Unfortunately it handles common symbols in a
+non-standard way (it doesn't subtract the old value) but that's a
+different story (we can't change it without losing backward
+compatibility with old object files) (coff-i386 does subtract the old
+value, to be compatible with existing coff-i386 targets, like SCO).
+
+So everything works fine when not producing relocateable output.  When
+we are producing relocateable output, logically we should do exactly
+what we do when not producing relocateable output.  Therefore, your
+patch is correct.  In fact, it should probably always just set
+reloc_entry->addend to 0 for all cases, since it is, in fact, going to
+add the value into the object file.  This won't hurt the COFF code,
+which doesn't use the addend; I'm not sure what it will do to other
+formats (the thing to check for would be whether any formats both use
+the addend and set partial_inplace).
+
+When I wanted to make coff-i386 produce relocateable output, I ran
+into the problem that you are running into: I wanted to remove that
+line.  Rather than risk it, I made the coff-i386 relocs use a special
+function; it's coff_i386_reloc in coff-i386.c.  The function
+specifically adds the addend field into the object file, knowing that
+bfd_perform_relocation is not going to.  If you remove that line, then
+coff-i386.c will wind up adding the addend field in twice.  It's
+trivial to fix; it just needs to be done.
+
+The problem with removing the line is just that it may break some
+working code.  With BFD it's hard to be sure of anything.  The right
+way to deal with this is simply to build and test at least all the
+supported COFF targets.  It should be straightforward if time and disk
+space consuming.  For each target:
+    1) build the linker
+    2) generate some executable, and link it using -r (I would
+       probably use paranoia.o and link against newlib/libc.a, which
+       for all the supported targets would be available in
+       /usr/cygnus/progressive/H-host/target/lib/libc.a).
+    3) make the change to reloc.c
+    4) rebuild the linker
+    5) repeat step 2
+    6) if the resulting object files are the same, you have at least
+       made it no worse
+    7) if they are different you have to figure out which version is
+       right
+*/
+	      relocation -= reloc_entry->addend;
+#endif
+	      reloc_entry->addend = 0;
+	    }
+	  else
+	    {
+	      reloc_entry->addend = relocation;
+	    }
+	}
+    }
+  else 
+    {
+      reloc_entry->addend = 0;
+    }
+
+  /* FIXME: This overflow checking is incomplete, because the value
+     might have overflowed before we get here.  For a correct check we
+     need to compute the value in a size larger than bitsize, but we
+     can't reasonably do that for a reloc the same size as a host
+     machine word.
+     FIXME: We should also do overflow checking on the result after
+     adding in the value contained in the object file.  */
+  if (howto->complain_on_overflow != complain_overflow_dont)
+    {
+      bfd_vma check;
+
+      /* Get the value that will be used for the relocation, but
+	 starting at bit position zero.  */
+      if (howto->rightshift > howto->bitpos)
+	check = relocation >> (howto->rightshift - howto->bitpos);
+      else
+	check = relocation << (howto->bitpos - howto->rightshift);
+      switch (howto->complain_on_overflow)
+	{
+	case complain_overflow_signed:
+	  {
+	    /* Assumes two's complement.  */
+	    bfd_signed_vma reloc_signed_max = (1 << (howto->bitsize - 1)) - 1;
+	    bfd_signed_vma reloc_signed_min = ~ reloc_signed_max;
+
+	    /* The above right shift is incorrect for a signed value.
+	       Fix it up by forcing on the upper bits.  */
+	    if (howto->rightshift > howto->bitpos
+		&& (bfd_signed_vma) relocation < 0)
+	      check |= ((bfd_vma) -1
+			&~ ((bfd_vma) -1
+			    >> (howto->rightshift - howto->bitpos)));
+	    if ((bfd_signed_vma) check > reloc_signed_max
+		|| (bfd_signed_vma) check < reloc_signed_min)
+	      flag = bfd_reloc_overflow;
+	  }
+	  break;
+	case complain_overflow_unsigned:
+	  {
+	    /* Assumes two's complement.  This expression avoids
+	       overflow if howto->bitsize is the number of bits in
+	       bfd_vma.  */
+	    bfd_vma reloc_unsigned_max =
+	      (((1 << (howto->bitsize - 1)) - 1) << 1) | 1;
+
+	    if ((bfd_vma) check > reloc_unsigned_max)
+	      flag = bfd_reloc_overflow;
+	  }
+	  break;
+	case complain_overflow_bitfield:
+	  {
+	    /* Assumes two's complement.  This expression avoids
+	       overflow if howto->bitsize is the number of bits in
+	       bfd_vma.  */
+	    bfd_vma reloc_bits = (((1 << (howto->bitsize - 1)) - 1) << 1) | 1;
+
+	    if (((bfd_vma) check &~ reloc_bits) != 0
+		&& ((bfd_vma) check &~ reloc_bits) != (-1 &~ reloc_bits))
+	      {
+		/* The above right shift is incorrect for a signed
+		   value.  See if turning on the upper bits fixes the
+		   overflow.  */
+		if (howto->rightshift > howto->bitpos
+		    && (bfd_signed_vma) relocation < 0)
+		  {
+		    check |= ((bfd_vma) -1
+			      &~ ((bfd_vma) -1
+				  >> (howto->rightshift - howto->bitpos)));
+		    if (((bfd_vma) check &~ reloc_bits) != (-1 &~ reloc_bits))
+		      flag = bfd_reloc_overflow;
+		  }
+		else
+		  flag = bfd_reloc_overflow;
+	      }
+	  }
+	  break;
+	default:
+	  abort ();
+	}
+    }
+  
   /* 
     Either we are relocating all the way, or we don't want to apply
     the relocation to the reloc entry (probably because there isn't
     any room in the output format to describe addends to relocs)
     */
-  relocation >>= howto->rightshift;
+
+  /* The cast to bfd_vma avoids a bug in the Alpha OSF/1 C compiler
+     (OSF version 1.3, compiler version 3.11).  It miscompiles the
+     following program:
+
+     struct str
+     {
+       unsigned int i0;
+     } s = { 0 };
+
+     int
+     main ()
+     {
+       unsigned long x;
+
+       x = 0x100000000;
+       x <<= (unsigned long) s.i0;
+       if (x == 0)
+	 printf ("failed\n");
+       else
+	 printf ("succeeded (%lx)\n", x);
+     }
+     */
+
+  relocation >>= (bfd_vma) howto->rightshift;
 
   /* Shift everything up to where it's going to be used */
    
-  relocation <<= howto->bitpos;
+  relocation <<= (bfd_vma) howto->bitpos;
 
   /* Wait for the day when all have the mask in them */
 
@@ -647,9 +869,21 @@ DEFUN(bfd_perform_relocation,(abfd,
        break;
 
      case 3:
-
        /* Do nothing */
        break;
+
+     case 4:
+#ifdef BFD64
+       if (relocation)
+	 {
+	   bfd_vma x = bfd_get_64 (abfd, (bfd_byte *) data + addr);
+	   DOIT (x);
+	   bfd_put_64 (abfd, x, (bfd_byte *) data + addr);
+	 }
+#else
+       abort ();
+#endif
+       break;
      default:
        return bfd_reloc_other;
      }
@@ -678,56 +912,62 @@ TYPEDEF
 	bfd_reloc_code_type
 
 DESCRIPTION
-	The insides of a reloc code
+	The insides of a reloc code.  The idea is that, eventually, there
+	will be one enumerator for every type of relocation we ever do.
+	Pass one of these values to <<bfd_reloc_type_lookup>>, and it'll
+	return a howto pointer.
+
+	This does mean that the application must determine the correct
+	enumerator value; you can't get a howto pointer from a random set
+	of attributes.
 
 CODE_FRAGMENT
 .
 .typedef enum bfd_reloc_code_real 
-.
 .{
-.       {* 16 bits wide, simple reloc *}
+.  {* Basic absolute relocations *}
+.  BFD_RELOC_64,
+.  BFD_RELOC_32,
 .  BFD_RELOC_16,        
-.
-.       {* 8 bits wide, but used to form an address like 0xffnn *}
-.  BFD_RELOC_8_FFnn,
-.
-.       {* 8 bits wide, simple *}
 .  BFD_RELOC_8,
 .
-.       {* 8 bits wide, pc relative *}
+.  {* PC-relative relocations *}
+.  BFD_RELOC_64_PCREL,
+.  BFD_RELOC_32_PCREL,
+.  BFD_RELOC_24_PCREL,    {* used by i960 *}
+.  BFD_RELOC_16_PCREL,
 .  BFD_RELOC_8_PCREL,
 .
-.       {* The type of reloc used to build a contructor table - at the
-.          moment probably a 32 bit wide abs address, but the cpu can
-.          choose. *}
+.  {* Linkage-table relative *}
+.  BFD_RELOC_32_BASEREL,
+.  BFD_RELOC_16_BASEREL,
+.  BFD_RELOC_8_BASEREL,
 .
+.  {* The type of reloc used to build a contructor table - at the moment
+.     probably a 32 bit wide abs address, but the cpu can choose. *}
 .  BFD_RELOC_CTOR,
 .
-.       {* 32 bits wide, simple reloc *}
-.  BFD_RELOC_32,
-.	{* 32 bits, PC-relative *}
-.  BFD_RELOC_32_PCREL,
+.  {* 8 bits wide, but used to form an address like 0xffnn *}
+.  BFD_RELOC_8_FFnn,
 .
-.	{* High 22 bits of 32-bit value; simple reloc.  *}
+.  {* 32-bit pc-relative, shifted right 2 bits (i.e., 30-bit
+.     word displacement, e.g. for SPARC) *}
+.  BFD_RELOC_32_PCREL_S2,
+.
+.  {* High 22 bits of 32-bit value, placed into lower 22 bits of
+.     target word; simple reloc.  *}
 .  BFD_RELOC_HI22,
-.	{* Low 10 bits.  *}
+.  {* Low 10 bits.  *}
 .  BFD_RELOC_LO10,
 .
-.	{* Reloc types used for i960/b.out.  *}
-.  BFD_RELOC_24_PCREL,
+.  {* Reloc types used for i960/b.out.  *}
 .  BFD_RELOC_I960_CALLJ,
 .
-.  BFD_RELOC_16_PCREL,
-.	{* 32-bit pc-relative, shifted right 2 bits (i.e., 30-bit
-.	   word displacement, e.g. for SPARC) *}
-.  BFD_RELOC_32_PCREL_S2,
-.
 .  {* now for the sparc/elf codes *}
 .  BFD_RELOC_NONE,		{* actually used *}
 .  BFD_RELOC_SPARC_WDISP22,
 .  BFD_RELOC_SPARC22,
 .  BFD_RELOC_SPARC13,
-.  BFD_RELOC_SPARC_BASE13,
 .  BFD_RELOC_SPARC_GOT10,
 .  BFD_RELOC_SPARC_GOT13,
 .  BFD_RELOC_SPARC_GOT22,
@@ -740,33 +980,152 @@ CODE_FRAGMENT
 .  BFD_RELOC_SPARC_RELATIVE,
 .  BFD_RELOC_SPARC_UA32,
 .
-.  {* this one is a.out specific? *}
+.  {* these are a.out specific? *}
+.  BFD_RELOC_SPARC_BASE13,
 .  BFD_RELOC_SPARC_BASE22,
 .
-.       {* Bits 27..2 of the relocation address shifted right 2 bits;
-.         simple reloc otherwise.  *}
+.
+.  {* Bits 27..2 of the relocation address shifted right 2 bits;
+.     simple reloc otherwise.  *}
 .  BFD_RELOC_MIPS_JMP,
 .
-.       {* signed 16-bit pc-relative, shifted right 2 bits (e.g. for MIPS) *}
+.  {* signed 16-bit pc-relative, shifted right 2 bits (e.g. for MIPS) *}
 .  BFD_RELOC_16_PCREL_S2,
 .
-.       {* High 16 bits of 32-bit value; simple reloc.  *}
+.  {* High 16 bits of 32-bit value; simple reloc.  *}
 .  BFD_RELOC_HI16,
-.       {* High 16 bits of 32-bit value but the low 16 bits will be sign
-.          extended and added to form the final result.  If the low 16
-.          bits form a negative number, we need to add one to the high value
-.          to compensate for the borrow when the low bits are added.  *}
+.  {* High 16 bits of 32-bit value but the low 16 bits will be sign
+.     extended and added to form the final result.  If the low 16
+.     bits form a negative number, we need to add one to the high value
+.     to compensate for the borrow when the low bits are added.  *}
 .  BFD_RELOC_HI16_S,
-.       {* Low 16 bits.  *}
+.  {* Low 16 bits.  *}
 .  BFD_RELOC_LO16,
 .
+.  {* 16 bit relocation relative to the global pointer.  *}
+.  BFD_RELOC_MIPS_GPREL,
+.
+.  {* Relocation against a MIPS literal section.  *}
+.  BFD_RELOC_MIPS_LITERAL,
+.
+.  {* MIPS ELF relocations.  *}
+.  BFD_RELOC_MIPS_GOT16,
+.  BFD_RELOC_MIPS_CALL16,
+.  BFD_RELOC_MIPS_GPREL32,
+.
+.  {* These are, so far, specific to HPPA processors.  I'm not sure that some
+.     don't duplicate other reloc types, such as BFD_RELOC_32 and _32_PCREL.
+.     Also, many more were in the list I got that don't fit in well in the
+.     model BFD uses, so I've omitted them for now.  If we do make this reloc
+.     type get used for code that really does implement the funky reloc types,
+.     they'll have to be added to this list.  *}
+.  BFD_RELOC_HPPA_32,
+.  BFD_RELOC_HPPA_11,
+.  BFD_RELOC_HPPA_14,
+.  BFD_RELOC_HPPA_17,
+.
+.  BFD_RELOC_HPPA_L21,
+.  BFD_RELOC_HPPA_R11,
+.  BFD_RELOC_HPPA_R14,
+.  BFD_RELOC_HPPA_R17,
+.  BFD_RELOC_HPPA_LS21,
+.  BFD_RELOC_HPPA_RS11,
+.  BFD_RELOC_HPPA_RS14,
+.  BFD_RELOC_HPPA_RS17,
+.  BFD_RELOC_HPPA_LD21,
+.  BFD_RELOC_HPPA_RD11,
+.  BFD_RELOC_HPPA_RD14,
+.  BFD_RELOC_HPPA_RD17,
+.  BFD_RELOC_HPPA_LR21,
+.  BFD_RELOC_HPPA_RR14,
+.  BFD_RELOC_HPPA_RR17,
+.
+.  BFD_RELOC_HPPA_GOTOFF_11,
+.  BFD_RELOC_HPPA_GOTOFF_14,
+.  BFD_RELOC_HPPA_GOTOFF_L21,
+.  BFD_RELOC_HPPA_GOTOFF_R11,
+.  BFD_RELOC_HPPA_GOTOFF_R14,
+.  BFD_RELOC_HPPA_GOTOFF_LS21,
+.  BFD_RELOC_HPPA_GOTOFF_RS11,
+.  BFD_RELOC_HPPA_GOTOFF_RS14,
+.  BFD_RELOC_HPPA_GOTOFF_LD21,
+.  BFD_RELOC_HPPA_GOTOFF_RD11,
+.  BFD_RELOC_HPPA_GOTOFF_RD14,
+.  BFD_RELOC_HPPA_GOTOFF_LR21,
+.  BFD_RELOC_HPPA_GOTOFF_RR14,
+.
+.  BFD_RELOC_HPPA_DLT_32,
+.  BFD_RELOC_HPPA_DLT_11,
+.  BFD_RELOC_HPPA_DLT_14,
+.  BFD_RELOC_HPPA_DLT_L21,
+.  BFD_RELOC_HPPA_DLT_R11,
+.  BFD_RELOC_HPPA_DLT_R14,
+.
+.  BFD_RELOC_HPPA_ABS_CALL_11,
+.  BFD_RELOC_HPPA_ABS_CALL_14,
+.  BFD_RELOC_HPPA_ABS_CALL_17,
+.  BFD_RELOC_HPPA_ABS_CALL_L21,
+.  BFD_RELOC_HPPA_ABS_CALL_R11,
+.  BFD_RELOC_HPPA_ABS_CALL_R14,
+.  BFD_RELOC_HPPA_ABS_CALL_R17,
+.  BFD_RELOC_HPPA_ABS_CALL_LS21,
+.  BFD_RELOC_HPPA_ABS_CALL_RS11,
+.  BFD_RELOC_HPPA_ABS_CALL_RS14,
+.  BFD_RELOC_HPPA_ABS_CALL_RS17,
+.  BFD_RELOC_HPPA_ABS_CALL_LD21,
+.  BFD_RELOC_HPPA_ABS_CALL_RD11,
+.  BFD_RELOC_HPPA_ABS_CALL_RD14,
+.  BFD_RELOC_HPPA_ABS_CALL_RD17,
+.  BFD_RELOC_HPPA_ABS_CALL_LR21,
+.  BFD_RELOC_HPPA_ABS_CALL_RR14,
+.  BFD_RELOC_HPPA_ABS_CALL_RR17,
+.
+.  BFD_RELOC_HPPA_PCREL_CALL_11,
+.  BFD_RELOC_HPPA_PCREL_CALL_12,
+.  BFD_RELOC_HPPA_PCREL_CALL_14,
+.  BFD_RELOC_HPPA_PCREL_CALL_17,
+.  BFD_RELOC_HPPA_PCREL_CALL_L21,
+.  BFD_RELOC_HPPA_PCREL_CALL_R11,
+.  BFD_RELOC_HPPA_PCREL_CALL_R14,
+.  BFD_RELOC_HPPA_PCREL_CALL_R17,
+.  BFD_RELOC_HPPA_PCREL_CALL_LS21,
+.  BFD_RELOC_HPPA_PCREL_CALL_RS11,
+.  BFD_RELOC_HPPA_PCREL_CALL_RS14,
+.  BFD_RELOC_HPPA_PCREL_CALL_RS17,
+.  BFD_RELOC_HPPA_PCREL_CALL_LD21,
+.  BFD_RELOC_HPPA_PCREL_CALL_RD11,
+.  BFD_RELOC_HPPA_PCREL_CALL_RD14,
+.  BFD_RELOC_HPPA_PCREL_CALL_RD17,
+.  BFD_RELOC_HPPA_PCREL_CALL_LR21,
+.  BFD_RELOC_HPPA_PCREL_CALL_RR14,
+.  BFD_RELOC_HPPA_PCREL_CALL_RR17,
+.
+.  BFD_RELOC_HPPA_PLABEL_32,
+.  BFD_RELOC_HPPA_PLABEL_11,
+.  BFD_RELOC_HPPA_PLABEL_14,
+.  BFD_RELOC_HPPA_PLABEL_L21,
+.  BFD_RELOC_HPPA_PLABEL_R11,
+.  BFD_RELOC_HPPA_PLABEL_R14,
+.
+.  BFD_RELOC_HPPA_UNWIND_ENTRY,
+.  BFD_RELOC_HPPA_UNWIND_ENTRIES,
+.
+.  {* i386/elf relocations *}
+.  BFD_RELOC_386_GOT32,
+.  BFD_RELOC_386_PLT32,
+.  BFD_RELOC_386_COPY,
+.  BFD_RELOC_386_GLOB_DAT,
+.  BFD_RELOC_386_JUMP_SLOT,
+.  BFD_RELOC_386_RELATIVE,
+.  BFD_RELOC_386_GOTOFF,
+.  BFD_RELOC_386_GOTPC,
+.
 .  {* this must be the highest numeric value *}
 .  BFD_RELOC_UNUSED
 . } bfd_reloc_code_real_type;
 */
 
 
-
 /*
 SECTION
 	bfd_reloc_type_lookup
@@ -792,7 +1151,7 @@ DEFUN(bfd_reloc_type_lookup,(abfd, code),
 }
 
 static reloc_howto_type bfd_howto_32 =
- HOWTO(0, 00,2,32,false,0,false,true,0,"VRT32", false,0xffffffff,0xffffffff,true);
+ HOWTO(0, 00,2,32,false,0,complain_overflow_bitfield,0,"VRT32", false,0xffffffff,0xffffffff,true);
 
 
 /*
@@ -801,7 +1160,7 @@ INTERNAL_FUNCTION
 
 SYNOPSIS
 	CONST struct reloc_howto_struct *bfd_default_reloc_type_lookup
-	(CONST struct bfd_arch_info *,
+	(bfd *abfd AND
          bfd_reloc_code_real_type  code);
 
 DESCRIPTION
@@ -809,17 +1168,18 @@ DESCRIPTION
 
 
 */
+
 CONST struct reloc_howto_struct *
-DEFUN(bfd_default_reloc_type_lookup,(arch,  code),
-     CONST struct bfd_arch_info *arch AND
-      bfd_reloc_code_real_type  code)
+DEFUN(bfd_default_reloc_type_lookup, (abfd, code),
+      bfd *abfd AND
+      bfd_reloc_code_real_type code)
 {
   switch (code) 
     {
     case BFD_RELOC_CTOR:
       /* The type of reloc used in a ctor, which will be as wide as the
-	 address - so either a 64, 32, or 16 bitter.. */
-      switch (arch->bits_per_address) {
+	 address - so either a 64, 32, or 16 bitter.  */
+      switch (bfd_get_arch_info (abfd)->bits_per_address) {
       case 64:
 	BFD_FAIL();
       case 32:
@@ -898,8 +1258,7 @@ DEFUN(bfd_generic_get_relocated_section_contents,(abfd,
 
 
 
-  bfd_size_type reloc_size = bfd_get_reloc_upper_bound(input_bfd,
-						       input_section);
+  size_t reloc_size = bfd_get_reloc_upper_bound(input_bfd, input_section);
   arelent **reloc_vector = (arelent **) alloca(reloc_size);
   
   /* read in the section */
