@@ -26,12 +26,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
    o - We also handle core files.
    o - We also handle archives.
    If you write shell scripts which manipulate this info then you may be
-   out of luck; there's no +predantic switch.
+   out of luck; there's no --predantic option.
 */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "getopt.h"
+#include "bucomm.h"
 
 #ifndef BSD_DEFAULT
 #define BSD_DEFAULT 1
@@ -48,7 +49,6 @@ int return_code = 0;
 
 /* IMPORTS */
 extern char *program_version;
-extern char *program_name;
 extern char *target;
 
 /* Forward declarations */
@@ -58,37 +58,36 @@ display_file PARAMS ((char *filename));
 
 static void
 print_sizes PARAMS ((bfd *file));
+
+static void
+berkeley_sum PARAMS ((bfd *, sec_ptr, PTR));
 
 /** main and like trivia */
 
 void
-usage ()
+usage (stream, status)
+     FILE *stream;
+     int status;
 {
-  fprintf (stderr, "size %s\nUsage: %s -{dox}{AB}V files ...\n",
-	 program_version, program_name);
-  fputs("\t+radix={8|10|16} -- select appropriate output radix.\n\
-\t-d -- output in decimal\n\
-\t-o -- output in octal\n\
-\t-x -- output in hex", stderr);
-  fputs("\t+format={Berkeley|SysV} -- select display format.\n\
-\t-A -- SysV(AT&T) format\n\
-\t-B -- BSD format", stderr);
+  fprintf (stream, "\
+Usage: %s [-ABdoxV] [--format=berkeley|sysv] [--radix=8|10|16]\n\
+       [--target=bfdname] [--version] [--help] [file...]\n", program_name);
 #if BSD_DEFAULT
-  fputs("\t  (Default is +format=Berkeley)", stderr);
+  fputs ("       (default is --format=berkeley)\n", stream);
 #else
-  fputs("\t  (Default is +format=SysV)", stderr);
+  fputs ("       (default is --format=sysv)\n", stream);
 #endif
-  fputs("\t-V, +version -- display program version, etc.\n\
-\t+help -- this message\n", stderr);
-  exit(1);
+  exit (status);
 }
 
-struct option long_options[] = {{"radix",   no_argument, 0, 0},
-				{"format",  required_argument, 0, 0},
-				{"version", no_argument, &show_version, 1},
-				{"target",  optional_argument, NULL, 0},
-				{"help",    no_argument, &show_help, 1},
-				{0, no_argument, 0, 0}};
+struct option long_options[] = {
+  {"format",  required_argument, 0, 200},
+  {"radix",   required_argument, 0, 201},
+  {"target",  required_argument, 0, 202},
+  {"version", no_argument, &show_version, 1},
+  {"help",    no_argument, &show_help, 1},
+  {0, no_argument, 0, 0}
+};
 
 int
 main (argc, argv)
@@ -97,32 +96,29 @@ main (argc, argv)
 {
   int temp;
   int c;			/* sez which option char */
-  int option_index = 0;
   extern int optind;		/* steps thru options */
+
   program_name = *argv;
 
   bfd_init();
 
   while ((c = getopt_long(argc, argv, "ABVdox", long_options,
-			  &option_index)) != EOF)
+			  (int *) 0)) != EOF)
     switch(c) {
-    case 0:
-      if (!strcmp("format",(long_options[option_index]).name)) {
+    case 200:			/* --format */
 	switch(*optarg) {
 	case 'B': case 'b': berkeley_format = 1; break;
 	case 'S': case 's': berkeley_format = 0; break;
-	default: printf("Unknown option to +format: %s\n", optarg);
-	  usage();
+	default: fprintf(stderr, "invalid argument to --format: %s\n", optarg);
+	  usage(stderr, 1);
 	}
 	break;
-      }
 
-      if (!strcmp("target",(long_options[option_index]).name)) {
+      case 202:			/* --target */
 	target = optarg;
 	break;
-      }
 
-      if (!strcmp("radix",(long_options[option_index]).name)) {
+      case 201:			/* --radix */
 #ifdef ANSI_LIBRARIES
 	temp = strtol(optarg, NULL, 10);
 #else
@@ -133,21 +129,26 @@ main (argc, argv)
 	case 8:  radix = octal; break;
 	case 16: radix = hex; break;
 	default: printf("Unknown radix: %s\n", optarg);
-	  usage();
+	  usage(stderr, 1);
 	}
-      }
-      break;
+	break;
+
     case 'A': berkeley_format = 0; break;
     case 'B': berkeley_format = 1; break;
     case 'V': show_version = 1; break;
     case 'd': radix = decimal; break;
     case 'x': radix = hex; break;
     case 'o': radix = octal; break;
-    case '?': usage();
+    case 0: break;
+    case '?': usage(stderr, 1);
     }
 
-  if (show_version) printf("%s version %s\n", program_name, program_version);
-  if (show_help) usage();
+  if (show_version) {
+    printf("GNU %s version %s\n", program_name, program_version);
+    exit (0);
+  }
+  if (show_help)
+    usage(stdout, 0);
 	
   if (optind == argc)
     display_file ("a.out");
@@ -202,6 +203,7 @@ display_file(filename)
 
   file = bfd_openr (filename, target);
   if (file == NULL) {
+    fprintf (stderr, "%s: ", program_name);
     bfd_perror (filename);
     return_code = 1;
     return;
@@ -215,6 +217,7 @@ display_file(filename)
        arfile = bfd_openr_next_archived_file (file, arfile);
       if (arfile == NULL) {
 	if (bfd_error != no_more_archived_files) {
+	  fprintf (stderr, "%s: ", program_name);
 	  bfd_perror (bfd_get_filename (file));
 	  return_code = 2;
         }
@@ -252,45 +255,38 @@ rprint_number(width, num)
 	  width, (unsigned long)num);
 }
 
-static char *bss_section_name = ".bss";
-static char *data_section_name = ".data";
-static char *stack_section_name = ".stack";
-static char *text_section_name = ".text";
+static bfd_size_type bsssize;
+static bfd_size_type datasize;
+static bfd_size_type textsize;
+
+static void
+berkeley_sum (abfd, sec, ignore)
+     bfd *abfd;
+     sec_ptr sec;
+     PTR ignore;
+{
+  bfd_size_type size;
+
+  size = bfd_get_section_size_before_reloc (sec);
+  if (bfd_get_section_flags (abfd, sec) & SEC_CODE)
+    textsize += size;
+  else if (bfd_get_section_flags (abfd, sec) & SEC_DATA)
+    datasize += size;
+  else if (bfd_get_section_flags (abfd, sec) & SEC_ALLOC)
+    bsssize += size;
+}
 
 void print_berkeley_format(abfd)
 bfd *abfd;
 {
   static int files_seen = 0;
-  sec_ptr bsssection = NULL;
-  sec_ptr datasection = NULL;
-  sec_ptr textsection = NULL;
-  bfd_size_type bsssize = 0;
-  bfd_size_type datasize = 0;
-  bfd_size_type textsize = 0;
-  bfd_size_type total = 0;
+  bfd_size_type total;
 
-  
-  if ((textsection = bfd_get_section_by_name (abfd, text_section_name))
-      != NULL) {
-    textsize = bfd_get_section_size_before_reloc (textsection);
-  }
+  bsssize = 0;
+  datasize = 0;
+  textsize = 0;
 
-  if ((datasection = bfd_get_section_by_name (abfd, data_section_name))
-      != NULL) {
-    datasize = bfd_get_section_size_before_reloc ( datasection);
-  }
-	
-  if (bfd_get_format (abfd) == bfd_object) {
-    if ((bsssection = bfd_get_section_by_name (abfd, bss_section_name))
-	!= NULL) {
-      bsssize = bfd_section_size(abfd, bsssection);
-    }
-  } else {
-    if ((bsssection = bfd_get_section_by_name (abfd, stack_section_name))
-	!= NULL) {
-      bsssize = bfd_section_size(abfd, bsssection);
-    }
-  }
+  bfd_map_over_sections (abfd, berkeley_sum, (PTR) NULL);
 
   if (files_seen++ == 0)
 #if 0	/* intel doesn't like bss/stk b/c they don't gave core files */
