@@ -39,24 +39,19 @@ contains a value, a section to which it is relative and a valid bit.
 #include "ldsym.h"
 #include "ldlang.h"
 
-extern char *output_filename;
-extern unsigned int undefined_global_sym_count;
-extern unsigned int defined_global_sym_count;
 extern bfd *output_bfd;
 extern bfd_size_type largest_section;
 extern lang_statement_list_type file_chain;
-extern args_type command_line;
 extern ld_config_type config;
 
 extern lang_input_statement_type *script_file;
-extern unsigned int defined_global_sym_count;
   extern lang_output_section_statement_type *abs_output_section;
 extern bfd_vma print_dot;
 
 
 static void
-DEFUN(exp_print_token,( code),
-      token_code_type code)
+exp_print_token (code)
+     token_code_type code;
 {
   static CONST struct  {
     token_code_type code;
@@ -118,20 +113,18 @@ DEFUN(exp_print_token,( code),
 }
 
 static void 
-DEFUN(make_abs,(ptr),
-      etree_value_type *ptr)
+make_abs (ptr)
+     etree_value_type *ptr;
 {
     asection *s = ptr->section->bfd_section;
     ptr->value += s->vma;
     ptr->section = abs_output_section;
 }
 
-static
-DEFUN(etree_value_type new_abs,(value),
-      bfd_vma value)
+static etree_value_type
+new_abs (value)
+     bfd_vma value;
 {
-
-  
   etree_value_type new;
   new.valid = true;
   new.section = abs_output_section;
@@ -140,22 +133,22 @@ DEFUN(etree_value_type new_abs,(value),
 }
 
 static void 
-DEFUN(check, (os, name, op),
-      lang_output_section_statement_type *os AND
-      CONST char *name AND
-      CONST char *op)
+check (os, name, op)
+     lang_output_section_statement_type *os;
+     CONST char *name;
+     CONST char *op;
 {
   if (os == (lang_output_section_statement_type *)NULL) {
-    einfo("%F%P %s uses undefined section %s\n", op, name);
+    einfo("%F%P: %s uses undefined section %s\n", op, name);
   }
   if (os->processed == false) {
-    einfo("%F%P %s forward reference of section %s\n",op, name);
+    einfo("%F%P: %s forward reference of section %s\n",op, name);
   }
 }
 
 etree_type *
-DEFUN(exp_intop,(value),
-      bfd_vma value)
+exp_intop (value)
+     bfd_vma value;
 {
   etree_type *new = (etree_type *)stat_alloc((bfd_size_type)(sizeof(new->value)));
   new->type.node_code = INT;
@@ -166,10 +159,10 @@ DEFUN(exp_intop,(value),
 }
 
 
-static
-DEFUN(etree_value_type new_rel,(value, section),
-      bfd_vma value AND
-      lang_output_section_statement_type *section)
+static etree_value_type
+new_rel (value, section)
+     bfd_vma value;
+     lang_output_section_statement_type *section;
 {
   etree_value_type new;
   new.valid = true;
@@ -178,11 +171,10 @@ DEFUN(etree_value_type new_rel,(value, section),
   return new;
 }
 
-static
-DEFUN(etree_value_type
-      new_rel_from_section, (value, section),
-      bfd_vma value AND
-      lang_output_section_statement_type *section)
+static etree_value_type
+new_rel_from_section (value, section)
+     bfd_vma value;
+     lang_output_section_statement_type *section;
 {
   etree_value_type new;
   new.valid = true;
@@ -195,12 +187,12 @@ DEFUN(etree_value_type
 }
 
 static etree_value_type 
-DEFUN(fold_binary,(tree, current_section, allocation_done, dot, dotp),
-      etree_type *tree AND
-      lang_output_section_statement_type *current_section AND
-      lang_phase_type  allocation_done AND
-      bfd_vma dot AND
-      bfd_vma *dotp)
+fold_binary (tree, current_section, allocation_done, dot, dotp)
+     etree_type *tree;
+     lang_output_section_statement_type *current_section;
+     lang_phase_type  allocation_done;
+     bfd_vma dot;
+     bfd_vma *dotp;
 {
   etree_value_type result;
 
@@ -227,7 +219,7 @@ DEFUN(fold_binary,(tree, current_section, allocation_done, dot, dotp),
 	  /* Mod,  both absolule*/
 
 	  if (other.value == 0) {
-	    einfo("%F%S % by zero\n");
+	    einfo("%F%S %% by zero\n");
 	  }
 	  result.value = (int)result.value % (int)other.value;
 	  break;
@@ -265,7 +257,7 @@ DEFUN(fold_binary,(tree, current_section, allocation_done, dot, dotp),
   return result;
 }
 etree_value_type 
-DEFUN_VOID(invalid)
+invalid ()
 {
   etree_value_type new;
   new.valid = false;
@@ -273,11 +265,11 @@ DEFUN_VOID(invalid)
 }
 
 etree_value_type 
-DEFUN(fold_name, (tree, current_section, allocation_done, dot),
-      etree_type *tree AND
-      lang_output_section_statement_type *current_section AND
-      lang_phase_type  allocation_done AND
-      bfd_vma dot)
+fold_name (tree, current_section, allocation_done, dot)
+     etree_type *tree;
+     lang_output_section_statement_type *current_section;
+     lang_phase_type  allocation_done;
+     bfd_vma dot;
 {
   etree_value_type result;
   switch (tree->type.node_code) 
@@ -346,7 +338,7 @@ DEFUN(fold_name, (tree, current_section, allocation_done, dot),
 	      }
 	    }
 	    if (result.valid == false) {
-	      einfo("%F%S: undefined symbol `%s' referenced in expression.\n",
+	      einfo("%F%S: undefined symbol `%s' referenced in expression\n",
 		   tree->name.name);
 	    }
 
@@ -387,13 +379,12 @@ DEFUN(fold_name, (tree, current_section, allocation_done, dot),
   return result;
 }
 etree_value_type 
-DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
-		    dot, dotp),
-      etree_type *tree AND
-      lang_output_section_statement_type *current_section AND
-      lang_phase_type  allocation_done AND
-      bfd_vma dot AND
-      bfd_vma *dotp)
+exp_fold_tree (tree, current_section, allocation_done, dot, dotp)
+     etree_type *tree;
+     lang_output_section_statement_type *current_section;
+     lang_phase_type  allocation_done;
+     bfd_vma dot;
+     bfd_vma *dotp;
 {
   etree_value_type result;
 
@@ -416,7 +407,7 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
 	{
 	 case ALIGN_K:
 	  if (allocation_done != lang_first_phase_enum) {
-	    result = new_rel_from_section(ALIGN(dot,
+	    result = new_rel_from_section(ALIGN_N(dot,
 						result.value) ,
 					  current_section);
 
@@ -440,7 +431,7 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
 		 */
 	      result.section = abs_output_section;
 	      result.value -= current_section->bfd_section->vma;
-	      result.valid = 1;
+	      result.valid = true;
 	    }
 	  }
 	  else 
@@ -464,7 +455,7 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
 	 case NEXT:
 	  if (allocation_done ==lang_allocating_phase_enum) {
 	    make_abs(&result);
-	    result.value = ALIGN(dot, result.value);
+	    result.value = ALIGN_N(dot, result.value);
 	  }
 	  else {
 	    /* Return next place aligned to value */
@@ -557,14 +548,14 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
 	  def->value = result.value;
 
 	  def->section = result.section->bfd_section;
-	  def->flags = BSF_GLOBAL | BSF_EXPORT;
+	  def->flags |= BSF_GLOBAL | BSF_EXPORT;
 
 
 	  def->udata = (PTR)NULL;
 	  def->name = sy->name;
 
 	  if (sy->sdefs_chain == 0)
-	   Q_enter_global_ref(def_ptr, sy->name);
+	   enter_global_ref(def_ptr, sy->name);
 	}
 
       }
@@ -575,7 +566,7 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
       result = fold_name(tree, current_section, allocation_done, dot);
       break;
      default:
-      einfo("%F%S Need more of these %d",tree->type.node_class );
+      einfo("%F%S need more of these %d\n",tree->type.node_class );
 
     }
   }
@@ -585,20 +576,20 @@ DEFUN(exp_fold_tree,(tree, current_section, allocation_done,
 
 
 etree_value_type 
-DEFUN(exp_fold_tree_no_dot,(tree, current_section, allocation_done),
-      etree_type *tree AND
-      lang_output_section_statement_type *current_section AND
-      lang_phase_type  allocation_done)
+exp_fold_tree_no_dot (tree, current_section, allocation_done)
+     etree_type *tree;
+     lang_output_section_statement_type *current_section;
+     lang_phase_type allocation_done;
 {
 return exp_fold_tree(tree, current_section, allocation_done, (bfd_vma)
 		     0, (bfd_vma *)NULL);
 }
 
 etree_type *
-DEFUN(exp_binop,(code, lhs, rhs),
-      int code AND
-      etree_type *lhs AND
-      etree_type *rhs)
+exp_binop (code, lhs, rhs)
+     int code;
+     etree_type *lhs;
+     etree_type *rhs;
 {
   etree_type value, *new;
   etree_value_type r;
@@ -620,11 +611,11 @@ DEFUN(exp_binop,(code, lhs, rhs),
 }
 
 etree_type *
-DEFUN(exp_trinop,(code, cond, lhs, rhs),
-      int code AND
-      etree_type *cond AND
-      etree_type *lhs AND
-      etree_type *rhs)
+exp_trinop (code, cond, lhs, rhs)
+     int code;
+     etree_type *cond;
+     etree_type *lhs;
+     etree_type *rhs;
 {
   etree_type value, *new;
   etree_value_type r;
@@ -645,9 +636,9 @@ DEFUN(exp_trinop,(code, cond, lhs, rhs),
 
 
 etree_type *
-DEFUN(exp_unop,(code, child),
-      int code AND
-      etree_type *child)
+exp_unop (code, child)
+     int code;
+     etree_type *child;
 {
   etree_type value, *new;
 
@@ -667,11 +658,10 @@ DEFUN(exp_unop,(code, child),
 
 
 etree_type *
-DEFUN(exp_nameop,(code, name),
-      int code AND
-      CONST char *name)
+exp_nameop (code, name)
+     int code;
+     CONST char *name;
 {
-
   etree_type value, *new;
   etree_value_type r;
   value.name.type.node_code = code;
@@ -695,10 +685,10 @@ DEFUN(exp_nameop,(code, name),
 
 
 etree_type *
-DEFUN(exp_assop,(code, dst, src),
-      int code AND
-      CONST char *dst AND
-      etree_type *src)
+exp_assop (code, dst, src)
+     int code;
+     CONST char *dst;
+     etree_type *src;
 {
   etree_type value, *new;
 
@@ -720,8 +710,8 @@ DEFUN(exp_assop,(code, dst, src),
 }
 
 void 
-DEFUN(exp_print_tree,(tree),
-      etree_type *tree)
+exp_print_tree (tree)
+     etree_type *tree;
 {
   switch (tree->type.node_class) {
   case etree_value:
@@ -790,11 +780,11 @@ DEFUN(exp_print_tree,(tree),
 
 
 bfd_vma
-DEFUN(exp_get_vma,(tree, def, name, allocation_done),
-      etree_type *tree AND
-      bfd_vma def AND
-      char *name AND
-      lang_phase_type allocation_done)
+exp_get_vma (tree, def, name, allocation_done)
+     etree_type *tree;
+      bfd_vma def;
+     char *name;
+     lang_phase_type allocation_done;
 {
   etree_value_type r;
 
@@ -803,7 +793,7 @@ DEFUN(exp_get_vma,(tree, def, name, allocation_done),
 		 abs_output_section,
 		      allocation_done);
     if (r.valid == false && name) {
-      einfo("%F%S Nonconstant expression for %s\n",name);
+      einfo("%F%S nonconstant expression for %s\n",name);
     }
     return r.value;
   }
@@ -813,11 +803,11 @@ DEFUN(exp_get_vma,(tree, def, name, allocation_done),
 }
 
 int 
-DEFUN(exp_get_value_int,(tree,def,name, allocation_done),
-      etree_type *tree AND
-      int def AND
-      char *name AND
-      lang_phase_type allocation_done)
+exp_get_value_int (tree,def,name, allocation_done)
+     etree_type *tree;
+     int def;
+     char *name;
+     lang_phase_type allocation_done;
 {
   return (int)exp_get_vma(tree,(bfd_vma)def,name, allocation_done);
 }
