@@ -264,7 +264,7 @@ struct memory_region_struct *
 
 void lang_map PARAMS ((void));
 void lang_set_flags PARAMS ((int *, CONST char *));
-void lang_add_output PARAMS ((CONST char *));
+void lang_add_output PARAMS ((CONST char *, int from_script));
 
 void lang_final PARAMS ((void));
 struct symbol_cache_entry *
@@ -336,7 +336,7 @@ lang_output_section_statement_type *
       lang_output_section_statement_lookup PARAMS ((CONST char * CONST name));
 
 void ldlang_add_undef PARAMS ((CONST char *CONST name));
-void lang_add_output_format PARAMS ((CONST char *));
+void lang_add_output_format PARAMS ((CONST char *, int from_script));
 
 
 void lang_list_init PARAMS ((lang_statement_list_type*));
