@@ -24,12 +24,10 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "ldemul.h"
 
 void
-DEFUN(ldversion,(noisy),
-      int noisy)
-
+ldversion (noisy)
+     int noisy;
 {
-
-  fprintf(stdout,"ld version 2.1 (with BFD %s)\n", BFD_VERSION);
+  fprintf(stdout,"ld version 2.2 (with BFD %s)\n", BFD_VERSION);
 
 
   if (noisy) 
@@ -45,3 +43,26 @@ DEFUN(ldversion,(noisy),
     }
   }
 }
+
+extern char *program_name;
+
+void
+help ()
+{
+  printf ("\
+Usage: %s [-o output] objfile...\n\
+       [-Aarchitecture] [-b input-format] [-Bstatic]\n\
+       [-c MRI-commandfile] [-d | -dc | -dp]\n\
+       [-defsym symbol=expression] [-e entry] [-F] [-F format]\n",
+	  program_name);
+  puts ("\
+       [-format input-format] [-g] [-G size] [--help] [-i]\n\
+       [-larchive] [-Lsearchdir] [-M] [-Map mapfile]\n\
+       [-m emulation] [-N | -n] [-noinhibit-exec]\n\
+       [-oformat output-format] [-R filename] [-relax]");
+  puts ("\
+       [-r | -Ur] [-S] [-s] [-sort-common] [-T commandfile]\n\
+       [-Ttext textorg] [-Tdata dataorg] [-Tbss bssorg] [-t]\n\
+       [-u symbol] [-V] [-v] [--version] [-warn-common]\n\
+       [-X] [-x] [-ysymbol]");
+}
