@@ -56,9 +56,9 @@ struct section_name_struct *subalignment;
 extern char *strdup();
 
 static struct section_name_struct **
-DEFUN(lookup,(name, list),
-CONST char *name AND
-struct section_name_struct **list)
+lookup (name, list)
+     CONST char *name;
+     struct section_name_struct **list;
 {
 
   struct section_name_struct **ptr = list;
@@ -79,13 +79,13 @@ struct section_name_struct **list)
 }
 
 static void
-DEFUN(mri_add_to_list,(list, name, vma, alias, align, subalign),
-      struct section_name_struct **list AND
-      CONST char *name AND
-      etree_type *vma AND
-      CONST char *alias AND
-      etree_type *align AND
-      etree_type *subalign)
+mri_add_to_list (list, name, vma, alias, align, subalign)
+     struct section_name_struct **list;
+     CONST char *name;
+     etree_type *vma;
+     CONST char *alias;
+     etree_type *align;
+     etree_type *subalign;
 {
   struct section_name_struct **ptr = lookup(name,list);
   (*ptr)->name = name;
@@ -99,10 +99,9 @@ DEFUN(mri_add_to_list,(list, name, vma, alias, align, subalign),
 
 
 void
-DEFUN(mri_output_section, (name, vma),
-      CONST char *name AND
-      etree_type *vma)
-
+mri_output_section (name, vma)
+     CONST char *name;
+     etree_type *vma;
 {
   mri_add_to_list(&address, name, vma, 0,0,0);
 }
@@ -110,24 +109,26 @@ DEFUN(mri_output_section, (name, vma),
 /* if any ABSOLUTE <name> are in the script, only load those files
 marked thus */
 
-void DEFUN(mri_only_load,(name), CONST char *name)
+void
+mri_only_load (name)
+     CONST char *name;
 {
   mri_add_to_list(&only_load, name, 0, 0,0,0);
 }
 
 
-etree_type *base;
+extern etree_type *base;
 
 void
-DEFUN(mri_base,(exp),
-      etree_type *exp)
+mri_base (exp)
+     etree_type *exp;
 {
   base = exp;
 }
 
 static int done_tree = 0;
 static void
-DEFUN_VOID(mri_draw_tree)
+mri_draw_tree ()
 {
   if (done_tree) return;
 
@@ -265,8 +266,8 @@ DEFUN_VOID(mri_draw_tree)
 
 }
 void
-DEFUN(mri_load,(name), 
-      CONST char *name)
+mri_load (name)
+     CONST char *name;
 {
   mri_draw_tree();
 
@@ -278,17 +279,17 @@ DEFUN(mri_load,(name),
 
 
 void
-DEFUN(mri_order,(name),
-      CONST char *name)
+mri_order (name)
+     CONST char *name;
 {
   mri_add_to_list(&order, name, 0, 0,0,0);
 }
 
 void 
-DEFUN(mri_alias, (want, is, isn ),
-      CONST char *want AND
-      CONST char *is AND
-      int isn)
+mri_alias (want, is, isn)
+     CONST char *want;
+     CONST char *is;
+     int isn;
 {
   if (!is) {
     /* Some sections are digits - */
@@ -302,29 +303,29 @@ DEFUN(mri_alias, (want, is, isn ),
 
 
 void 
-DEFUN(mri_name,(name),
-      CONST char *name)
+mri_name (name)
+     CONST char *name;
 {
-  lang_add_output(name);
+  lang_add_output(name, 1);
 
 }
 
 
 void
-DEFUN(mri_format,(name),
-      CONST char *name)
+mri_format (name)
+     CONST char *name;
 {
   if (strcmp(name, "S") == 0)
   {
-    lang_add_output_format("srec");
+    lang_add_output_format("srec", 1);
   }
   else if (strcmp(name, "IEEE") == 0)
   {
-    lang_add_output_format("ieee");
+    lang_add_output_format("ieee", 1);
   }
   else if (strcmp(name, "COFF") == 0)
   {
-    lang_add_output_format("coff-m68k");
+    lang_add_output_format("coff-m68k", 1);
   }
   else {
     einfo("%P%F: unknown format type %s\n", name);
@@ -333,34 +334,33 @@ DEFUN(mri_format,(name),
 
 
 void
-DEFUN(mri_public,(name, exp),
-      CONST char *name AND
-      etree_type *exp)
+mri_public (name, exp)
+     CONST char *name;
+     etree_type *exp;
 {
-
-lang_add_assignment(exp_assop('=', name, exp));
+  lang_add_assignment(exp_assop('=', name, exp));
 }
 
 void 
-DEFUN(mri_align,(name, exp),
-      CONST char *name AND
-      etree_type *exp)
+mri_align (name, exp)
+     CONST char *name;
+     etree_type *exp;
 {
   mri_add_to_list(&alignment, name,0,0,exp,0);
 }
 
 void 
-DEFUN(mri_alignmod,(name, exp),
-      CONST char *name AND
-      etree_type *exp)
+mri_alignmod (name, exp)
+     CONST char *name;
+     etree_type *exp;
 {
   mri_add_to_list(&subalignment, name,0,0,0,exp);
 }
 
 
 void 
-DEFUN(mri_truncate,(exp),
-      int exp)
+mri_truncate (exp)
+     int exp;
 {
   symbol_truncate = exp;
 }
